# Copyright (C) 2006-2007 by Dr. Dieter Maurer, Illstalst. 25, D-66571 Bubach, Germany
# see "LICENSE.txt" for details
#       $Id: __init__.py,v 1.1.1.1 2008/08/18 10:12:30 dieter Exp $
'''Profile enhancements.'''

# only 'Stats' is usefull for now
#from dmprofile import Profile

from dmstats import Stats
